@@ -28,7 +28,7 @@
  * surface and layer which groups surfaces. A unique ID whose type is integer is
  * required to create surface and layer. With the unique ID, surface and layer
  * are identified to control them. The API set consists of APIs to control
- * properties of surface and layers about followings,
+ * properties of surface and layers about the following:
  * - visibility.
  * - opacity.
  * - clipping (x,y,width,height).
@@ -43,9 +43,6 @@
  * way in In-Vehicle Infotainment system, which integrate several domains
  * in one system. A layer is allocated to a domain in order to control
  * application surfaces grouped to the layer all together.
- *
- * This API and ABI follow following specifications.
- * http://projects.genivi.org/wayland-ivi-extension/layer-manager-apis
  */
 
 #ifndef _IVI_LAYOUT_EXPORT_H_
@@ -56,17 +53,25 @@ extern "C" {
 #endif /* __cplusplus */
 
 #include <stdint.h>
+#include <limits.h>
 
 #include "stdbool.h"
-#include "compositor.h"
+#include <libweston/libweston.h>
+#include <libweston/plugin-registry.h>
 
 #define IVI_SUCCEEDED (0)
 #define IVI_FAILED (-1)
+#define IVI_INVALID_ID UINT_MAX
 
 struct ivi_layout_layer;
-struct ivi_layout_screen;
 struct ivi_layout_surface;
 
+enum ivi_layout_surface_type {
+	IVI_LAYOUT_SURFACE_TYPE_IVI,
+	IVI_LAYOUT_SURFACE_TYPE_DESKTOP,
+	IVI_LAYOUT_SURFACE_TYPE_INPUT_PANEL,
+};
+
 struct ivi_layout_surface_properties
 {
 	wl_fixed_t opacity;
@@ -87,6 +92,7 @@ struct ivi_layout_surface_properties
 	int32_t transition_type;
 	uint32_t transition_duration;
 	uint32_t event_mask;
+	enum ivi_layout_surface_type surface_type;
 };
 
 struct ivi_layout_layer_properties
@@ -101,7 +107,7 @@ struct ivi_layout_layer_properties
 	int32_t dest_width;
 	int32_t dest_height;
 	enum wl_output_transform orientation;
-	uint32_t visibility;
+	bool visibility;
 	int32_t transition_type;
 	uint32_t transition_duration;
 	double start_alpha;
@@ -110,6 +116,14 @@ struct ivi_layout_layer_properties
 	uint32_t event_mask;
 };
 
+struct ivi_layout_text_input_state
+{
+	bool overlay_panel;
+	struct ivi_layout_surface *surface;
+	struct ivi_layout_surface *input_panel;
+	pixman_box32_t cursor_rectangle;
+};
+
 enum ivi_layout_notification_mask {
 	IVI_NOTIFICATION_NONE        = 0,
 	IVI_NOTIFICATION_OPACITY     = (1 << 1),
@@ -140,6 +154,8 @@ enum ivi_layout_transition_type{
 	IVI_LAYOUT_TRANSITION_MAX,
 };
 
+#define IVI_LAYOUT_API_NAME "ivi_layout_api_v1"
+
 struct ivi_layout_interface {
 
 	/**
@@ -151,6 +167,14 @@ struct ivi_layout_interface {
 	 */
 	int32_t (*commit_changes)(void);
 
+	/**
+	 * \brief Rebuild view list without applying any new changes
+	 *
+	 * \return IVI_SUCCEEDED if the method call was successful
+	 * \return IVI_FAILED if the method call was failed
+	 */
+	int32_t (*commit_current)(void);
+
 	/**
 	 * surface controller interface
 	 */
@@ -163,7 +187,7 @@ struct ivi_layout_interface {
 	 * The pointer of the created ivi_surface is sent as the void *data argument
 	 * to the wl_listener::notify callback function of the listener.
 	 */
-	int32_t (*add_listener_create_surface)(struct wl_listener *listener);
+	void (*add_listener_create_surface)(struct wl_listener *listener);
 
 	/**
 	 * \brief add a listener for notification when ivi_surface is removed
@@ -173,7 +197,7 @@ struct ivi_layout_interface {
 	 * The pointer of the removed ivi_surface is sent as the void *data argument
 	 * to the wl_listener::notify callback function of the listener.
 	 */
-	int32_t (*add_listener_remove_surface)(struct wl_listener *listener);
+	void (*add_listener_remove_surface)(struct wl_listener *listener);
 
 	/**
 	 * \brief add a listener for notification when ivi_surface is configured
@@ -183,16 +207,23 @@ struct ivi_layout_interface {
 	 * The pointer of the configured ivi_surface is sent as the void *data argument
 	 * to the wl_listener::notify callback function of the listener.
 	 */
-	int32_t (*add_listener_configure_surface)(struct wl_listener *listener);
+	void (*add_listener_configure_surface)(struct wl_listener *listener);
+
+	/**
+	 * \brief add a listener for notification when desktop_surface is configured
+	 *
+	 * When an desktop_surface is configured, a signal is emitted
+	 * to the listening controller plugins.
+	 * The pointer of the configured desktop_surface is sent as the void *data argument
+	 * to the wl_listener::notify callback function of the listener.
+	 */
+	void (*add_listener_configure_desktop_surface)(struct wl_listener *listener);
 
 	/**
 	 * \brief Get all ivi_surfaces which are currently registered and managed
 	 * by the services
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*get_surfaces)(int32_t *pLength, struct ivi_layout_surface ***ppArray);
+	void (*get_surfaces)(int32_t *pLength, struct ivi_layout_surface ***ppArray);
 
 	/**
 	 * \brief get id of ivi_surface from ivi_layout_surface
@@ -224,65 +255,44 @@ struct ivi_layout_interface {
 	/**
 	 * \brief Get all Surfaces which are currently registered to a given
 	 * layer and are managed by the services
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*get_surfaces_on_layer)(struct ivi_layout_layer *ivilayer,
-					 int32_t *pLength,
-					 struct ivi_layout_surface ***ppArray);
+	void (*get_surfaces_on_layer)(struct ivi_layout_layer *ivilayer,
+				      int32_t *pLength,
+				      struct ivi_layout_surface ***ppArray);
 
 	/**
 	 * \brief Set the visibility of a ivi_surface.
 	 *
 	 * If a surface is not visible it will not be rendered.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*surface_set_visibility)(struct ivi_layout_surface *ivisurf,
-					  bool newVisibility);
+	void (*surface_set_visibility)(struct ivi_layout_surface *ivisurf,
+				       bool newVisibility);
 
 	/**
 	 * \brief Set the opacity of a surface.
 	 *
 	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
+	 * \return IVI_FAILED if the specified opacity is out of bounds
 	 */
 	int32_t (*surface_set_opacity)(struct ivi_layout_surface *ivisurf,
 				       wl_fixed_t opacity);
 
 	/**
 	 * \brief Set the area of a ivi_surface which should be used for the rendering.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*surface_set_source_rectangle)(struct ivi_layout_surface *ivisurf,
-						int32_t x, int32_t y,
-						int32_t width, int32_t height);
+	void (*surface_set_source_rectangle)(struct ivi_layout_surface *ivisurf,
+					     int32_t x, int32_t y,
+					     int32_t width, int32_t height);
 
 	/**
 	 * \brief Set the destination area of a ivi_surface within a ivi_layer
 	 * for rendering.
 	 *
 	 * The surface will be scaled to this rectangle for rendering.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*surface_set_destination_rectangle)(struct ivi_layout_surface *ivisurf,
-						     int32_t x, int32_t y,
-						     int32_t width, int32_t height);
-
-	/**
-	 * \brief Sets the orientation of a ivi_surface.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
-	 */
-	int32_t (*surface_set_orientation)(struct ivi_layout_surface *ivisurf,
-					   enum wl_output_transform orientation);
+	void (*surface_set_destination_rectangle)(struct ivi_layout_surface *ivisurf,
+						  int32_t x, int32_t y,
+						  int32_t width, int32_t height);
 
 	/**
 	 * \brief add a listener to listen property changes of ivi_surface
@@ -291,11 +301,8 @@ struct ivi_layout_interface {
 	 * signal is emitted to the listening controller plugins.
 	 * The pointer of the ivi_surface is sent as the void *data argument
 	 * to the wl_listener::notify callback function of the listener.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*surface_add_listener)(struct ivi_layout_surface *ivisurf,
+	void (*surface_add_listener)(struct ivi_layout_surface *ivisurf,
 					    struct wl_listener *listener);
 
 	/**
@@ -307,16 +314,21 @@ struct ivi_layout_interface {
 	/**
 	 * \brief set type of transition animation
 	 */
-	int32_t (*surface_set_transition)(struct ivi_layout_surface *ivisurf,
-					  enum ivi_layout_transition_type type,
-					  uint32_t duration);
+	void (*surface_set_transition)(struct ivi_layout_surface *ivisurf,
+				       enum ivi_layout_transition_type type,
+				       uint32_t duration);
 
 	/**
 	 * \brief set duration of transition animation
 	 */
-	int32_t (*surface_set_transition_duration)(
-					struct ivi_layout_surface *ivisurf,
-					uint32_t duration);
+	void (*surface_set_transition_duration)(struct ivi_layout_surface *ivisurf,
+						uint32_t duration);
+
+	/**
+	 * \brief set id of ivi_layout_surface
+	 */
+	int32_t (*surface_set_id)(struct ivi_layout_surface *ivisurf,
+				  uint32_t id_surface);
 
 	/**
 	 * layer controller interface
@@ -330,7 +342,7 @@ struct ivi_layout_interface {
 	 * The pointer of the created ivi_layer is sent as the void *data argument
 	 * to the wl_listener::notify callback function of the listener.
 	 */
-	int32_t (*add_listener_create_layer)(struct wl_listener *listener);
+	void (*add_listener_create_layer)(struct wl_listener *listener);
 
 	/**
 	 * \brief add a listener for notification when ivi_layer is removed
@@ -340,7 +352,7 @@ struct ivi_layout_interface {
 	 * The pointer of the removed ivi_layer is sent as the void *data argument
 	 * to the wl_listener::notify callback function of the listener.
 	 */
-	int32_t (*add_listener_remove_layer)(struct wl_listener *listener);
+	void (*add_listener_remove_layer)(struct wl_listener *listener);
 
 	/**
 	 * \brief Create a ivi_layer which should be managed by the service
@@ -361,11 +373,8 @@ struct ivi_layout_interface {
 	/**
 	 * \brief Get all ivi_layers which are currently registered and managed
 	 * by the services
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*get_layers)(int32_t *pLength, struct ivi_layout_layer ***ppArray);
+	void (*get_layers)(int32_t *pLength, struct ivi_layout_layer ***ppArray);
 
 	/**
 	 * \brief get id of ivi_layer from ivi_layout_layer
@@ -399,39 +408,30 @@ struct ivi_layout_interface {
 	 *
 	 * This means all the ivi-layers the ivi-surface was added to. It has
 	 * no relation to geometric overlaps.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*get_layers_under_surface)(struct ivi_layout_surface *ivisurf,
-					    int32_t *pLength,
+	void (*get_layers_under_surface)(struct ivi_layout_surface *ivisurf,
+					 int32_t *pLength,
 					    struct ivi_layout_layer ***ppArray);
 
 	/**
 	 * \brief Get all Layers of the given weston_output
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*get_layers_on_screen)(struct weston_output *output,
-					int32_t *pLength,
-					struct ivi_layout_layer ***ppArray);
+	void (*get_layers_on_screen)(struct weston_output *output,
+				     int32_t *pLength,
+				     struct ivi_layout_layer ***ppArray);
 
 	/**
 	 * \brief Set the visibility of a ivi_layer. If a ivi_layer is not visible,
 	 * the ivi_layer and its ivi_surfaces will not be rendered.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*layer_set_visibility)(struct ivi_layout_layer *ivilayer,
-					bool newVisibility);
+	void (*layer_set_visibility)(struct ivi_layout_layer *ivilayer,
+				     bool newVisibility);
 
 	/**
 	 * \brief Set the opacity of a ivi_layer.
 	 *
 	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
+	 * \return IVI_FAILED if the specified opacity is out of bounds
 	 */
 	int32_t (*layer_set_opacity)(struct ivi_layout_layer *ivilayer,
 				     wl_fixed_t opacity);
@@ -440,43 +440,25 @@ struct ivi_layout_interface {
 	 * \brief Set the area of a ivi_layer which should be used for the rendering.
 	 *
 	 * Only this part will be visible.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*layer_set_source_rectangle)(struct ivi_layout_layer *ivilayer,
-					      int32_t x, int32_t y,
-					      int32_t width, int32_t height);
+	void (*layer_set_source_rectangle)(struct ivi_layout_layer *ivilayer,
+					   int32_t x, int32_t y,
+					   int32_t width, int32_t height);
 
 	/**
 	 * \brief Set the destination area on the display for a ivi_layer.
 	 *
 	 * The ivi_layer will be scaled and positioned to this rectangle
 	 * for rendering
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*layer_set_destination_rectangle)(struct ivi_layout_layer *ivilayer,
-						   int32_t x, int32_t y,
-						   int32_t width, int32_t height);
-
-	/**
-	 * \brief Sets the orientation of a ivi_layer.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
-	 */
-	int32_t (*layer_set_orientation)(struct ivi_layout_layer *ivilayer,
-					 enum wl_output_transform orientation);
+	void (*layer_set_destination_rectangle)(struct ivi_layout_layer *ivilayer,
+						int32_t x, int32_t y,
+						int32_t width, int32_t height);
 
 	/**
 	 * \brief Add a ivi_surface to a ivi_layer which is currently managed by the service
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*layer_add_surface)(struct ivi_layout_layer *ivilayer,
+	void (*layer_add_surface)(struct ivi_layout_layer *ivilayer,
 				     struct ivi_layout_surface *addsurf);
 
 	/**
@@ -487,13 +469,10 @@ struct ivi_layout_interface {
 
 	/**
 	 * \brief Sets render order of ivi_surfaces within a ivi_layer
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*layer_set_render_order)(struct ivi_layout_layer *ivilayer,
-					  struct ivi_layout_surface **pSurface,
-					  int32_t number);
+	void (*layer_set_render_order)(struct ivi_layout_layer *ivilayer,
+				       struct ivi_layout_surface **pSurface,
+				       int32_t number);
 
 	/**
 	 * \brief add a listener to listen property changes of ivi_layer
@@ -502,19 +481,16 @@ struct ivi_layout_interface {
 	 * signal is emitted to the listening controller plugins.
 	 * The pointer of the ivi_layer is sent as the void *data argument
 	 * to the wl_listener::notify callback function of the listener.
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*layer_add_listener)(struct ivi_layout_layer *ivilayer,
-					  struct wl_listener *listener);
+	void (*layer_add_listener)(struct ivi_layout_layer *ivilayer,
+				   struct wl_listener *listener);
 
 	/**
 	 * \brief set type of transition animation
 	 */
-	int32_t (*layer_set_transition)(struct ivi_layout_layer *ivilayer,
-					enum ivi_layout_transition_type type,
-					uint32_t duration);
+	void (*layer_set_transition)(struct ivi_layout_layer *ivilayer,
+				     enum ivi_layout_transition_type type,
+				     uint32_t duration);
 
 	/**
 	 * screen controller interface
@@ -522,48 +498,39 @@ struct ivi_layout_interface {
 
 	/**
 	 * \brief Get the weston_outputs under the given ivi_layer
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*get_screens_under_layer)(struct ivi_layout_layer *ivilayer,
-					   int32_t *pLength,
-					   struct weston_output ***ppArray);
+	void (*get_screens_under_layer)(struct ivi_layout_layer *ivilayer,
+					int32_t *pLength,
+					struct weston_output ***ppArray);
 
 	/**
 	 * \brief Add a ivi_layer to a weston_output which is currently managed
 	 * by the service
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*screen_add_layer)(struct weston_output *output,
+	void (*screen_add_layer)(struct weston_output *output,
 				    struct ivi_layout_layer *addlayer);
 
 	/**
 	 * \brief Sets render order of ivi_layers on a weston_output
-	 *
-	 * \return IVI_SUCCEEDED if the method call was successful
-	 * \return IVI_FAILED if the method call was failed
 	 */
-	int32_t (*screen_set_render_order)(struct weston_output *output,
+	void (*screen_set_render_order)(struct weston_output *output,
 					   struct ivi_layout_layer **pLayer,
 					   const int32_t number);
 
 	/**
-	 * transision animation for layer
+	 * transition animation for layer
 	 */
 	void (*transition_move_layer_cancel)(struct ivi_layout_layer *layer);
-	int32_t (*layer_set_fade_info)(struct ivi_layout_layer* ivilayer,
-				       uint32_t is_fade_in,
-				       double start_alpha, double end_alpha);
+	void (*layer_set_fade_info)(struct ivi_layout_layer* ivilayer,
+				    uint32_t is_fade_in,
+				    double start_alpha, double end_alpha);
 
 	/**
 	 * surface content dumping for debugging
 	 */
-	int32_t (*surface_get_size)(struct ivi_layout_surface *ivisurf,
-				    int32_t *width, int32_t *height,
-				    int32_t *stride);
+	void (*surface_get_size)(struct ivi_layout_surface *ivisurf,
+				 int32_t *width, int32_t *height,
+				 int32_t *stride);
 
 	int32_t (*surface_dump)(struct weston_surface *surface,
 				void *target, size_t size,
@@ -578,8 +545,94 @@ struct ivi_layout_interface {
 	 */
 	struct ivi_layout_surface *
 		(*get_surface)(struct weston_surface *surface);
+
+	/**
+	 * \brief Remove a ivi_layer to a weston_output which is currently managed
+	 * by the service
+	 */
+	void (*screen_remove_layer)(struct weston_output *output,
+				    struct ivi_layout_layer *removelayer);
+
+	/**
+	 * \brief Add a shell destroy listener only once.
+	 *
+	 * The begining of shell destroying, this signal is emitted
+	 * to the listening controller plugins.
+	 * The null pointer sent as the void *data argument
+	 * to the wl_listener::notify callback function of the listener.
+	 *
+	 * \return IVI_SUCCEEDED if the method call was successful
+	 * \return IVI_FAILED if the method was called before
+	 */
+	int32_t (*shell_add_destroy_listener_once)(struct wl_listener *listener,
+				wl_notify_func_t destroy_handler);
+
+	/**
+	 * \brief add a listener for notification when input_panel_surface is
+	 * configured
+	 *
+	 * When an input_panel_surface is configured, a signal is emitted
+	 * to the listening controller plugins.
+	 * The pointer of the configured input_panel_surface is sent as the void
+	 * *data argument to the wl_listener::notify callback function of the
+	 * listener.
+	 */
+	void (*add_listener_configure_input_panel_surface)(struct wl_listener *listener);
+
+	/**
+	 * \brief add a listener for notification when an input_panel_surface
+	 * should be shown.
+	 *
+	 * When a client requests input panels, this signal is emitted for all
+	 * available input panels.
+	 * A pointer to a struct ivi_layout_text_input_state is sent as the void
+	 * *data argument to the wl_listener::notify callback function of the
+	 * listener.
+	 * It contains the surface that requested the input panel, the
+	 * input_panel_surface that should be shown and whether the input panel
+	 * is a toplevel o overlay panel.
+	 * For overlay panels, the relevant cursor_rectangle is filled with
+	 * coordinates relative to the client surface.
+	 */
+	void (*add_listener_show_input_panel)(struct wl_listener *listener);
+
+	/**
+	 * \brief add a listener for notification when an input_panel_surface
+	 * should be hidden.
+	 *
+	 * When a client requests that input panels are hidden, this signal is
+	 * emitted for all available input panels.
+	 * The pointer of the configured input_panel_surface is sent as the void
+	 * *data argument to the wl_listener::notify callback function of the
+	 * listener.
+	 */
+	void (*add_listener_hide_input_panel)(struct wl_listener *listener);
+
+	/**
+	 * \brief add a listener for notification when an input_panel_surface
+	 * should be updated.
+	 *
+	 * When the input panels need to be updated in some way, this signal is
+	 * called for available input panels.
+	 * This happens for example when the cursor_rectangle changes.
+	 * A pointer to a struct ivi_layout_text_input_state is sent as the void
+	 * *data argument to the wl_listener::notify callback function of the
+	 * listener.
+	 * See add_listener_show_input_panel for more details.
+	 */
+	void (*add_listener_update_input_panel)(struct wl_listener *listener);
 };
 
+static inline const struct ivi_layout_interface *
+ivi_layout_get_api(struct weston_compositor *compositor)
+{
+	const void *api;
+	api = weston_plugin_api_get(compositor, IVI_LAYOUT_API_NAME,
+				    sizeof(struct ivi_layout_interface));
+
+	return (const struct ivi_layout_interface *)api;
+}
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
